package designer.admin;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import com.ibm.mqe.registry.*;
import java.io.*;
import java.util.*;

/**
 * This class wrappers the local queue manager. 
 * The class exposes the published queue manager API.
 * The class provides several useful functions that augment the published API.  The most
 * useful of which are the abilities to create, start, and stop the queue manager.
 *
 * This class provides the ability to create a queue manager programatically without
 * using ini files or example code.  In order to do this a large number of defaults are
 * assumed.  These defaults can be overriden by placing values in a hashtable that is used 
 * as a start argument. Values in the hashtable are represented by string pairs.  The 
 * string used as a key gives the name of the attribute. The string stored against the
 * attribute name is its attribute value.  For simplicities sake, all parameter values
 * are strings.
 *
 * One of the values in the hashtable is required - the name of the queue manager.  The 
 * rest are optional.
 *
 * Starting the QueueManager
 * -------------------------
 * The queue manager base directory is always the current working directory for the JVM.
 * The queue manager is started in one of two modes: clean or not clean.  In clean mode
 * - Clean Mode: any existing file structure is deleted, and a new queue manager is 
 *               created.  This queue manager has three queues AdminQ, AdminReplyQueue
 *               and System.Default queue.
 * - Non-Clean:  The presence of an existing queue manager structure on disk, with the
 *               correct name/location is assumed.  This queue manager is started up
 *               with all pre-existing settings from the registry.
 *
 * Aliases:
 * --------
 * MQe uses substitution text in class resolution.  The standard aliases recognised by 
 * listed below, with their default values. These default values can be overriden by 
 * including an appropriate entry in the argument set
 * "FileRegistry",		"com.ibm.mqe.registry.MQeFileSession"
 * "MsgLog"				"com.ibm.mqe.adapters.MQeDiskFieldsAdapter"
 * "MemoryMsgLog"		"com.ibm.mqe.adapters.MQeMemoryFieldsAdapter"
 * "DefaultTransporter"	"com.ibm.mqe.MQeTransporter"));
 * "Network",			"com.ibm.mqe.adapters.MQeTcpipHttpAdapter"
 * "FastNetwork"		"com.ibm.mqe.adapters.MQeTcpipHistoryAdapter"
 * "DefaultChannel"		"com.ibm.mqe.MQeChannel"
 * "AttributeKey_2"		"com.ibm.mqe.attributes.MQeSharedKey"
 * "AttributeKey_1"		"com.ibm.mqe.MQeKey"
 * 
 * Creation date: (10/9/2000 2:40:46 PM)
 */
public class LocalQueueManager extends Object {
	public static final String RSP_QUEUEMANAGER_STARTED         = "RSP_SUCCESS QueueManager started";
	public static final String RSP_QUEUEMANAGER_ALREADYSTARTED     = "RSP_FAILURE QueueManager already started";
	public static final String RSP_QUEUEMANAGER_STOPPED         = "RSP_SUCCESS QueueManager stopped";
	public static final String RSP_QUEUEMANAGER_NOTRUNNING         = "RSP_FAILURE QueueManager not running";

	public static final String ARG_AGENT_PORT            = "port";
	public static final String ARG_QUEUEMANAGER_NAME  	= "queueManagerName";
	public static final String ARG_QUEUEMANAGER_PORT  	= "queueManagerPort";
	public static final String ARG_CLEANUP            	= "cleanup";
	public static final String ARG_PIN            		= "pin";

	
	protected static final String MY_ATTRIBUTE_RULES = "abstracttest.AttributeRule";
	private MQeQueueManager myQM = null;
	private MQeChannelListener channelListener;
	private String baseDirectory;

	private String port;
	private String queueManagerName;
	private boolean showInfo = false;
	private String key;


	private Hashtable arguments;
/**
 * Wrapper to create queue manager with given name.
 */
public LocalQueueManager() {
	super();
}
protected final void _comment(String s) {
	if (showInfo) {
		System.out.println("Comment: " + s);
	}
}
protected final void _comment(String s, Exception e) {
	if (showInfo) {
		System.out.println("Comment: " + s + "{" + e.getMessage() + "}");
	}
}
/**
 * Create the aliases neccessary for the creation of a queue manager
 *
 * Part of the programatic creation of a queue manager
 *
 */
private final void _createAliases() {
	try {
		MQe.alias("FileRegistry",       getArgument("FileRegistry",			"com.ibm.mqe.registry.MQeFileSession"));
		MQe.alias("MsgLog",             getArgument("MsgLog",				"com.ibm.mqe.adapters.MQeDiskFieldsAdapter"));
		MQe.alias("MemoryMsgLog",       getArgument("MemoryMsgLog",			"com.ibm.mqe.adapters.MQeMemoryFieldsAdapter"));
		MQe.alias("DefaultTransporter", getArgument("DefaultTransporter",	"com.ibm.mqe.MQeTransporter"));
		MQe.alias("Network",            getArgument("Network",				"com.ibm.mqe.adapters.MQeTcpipHttpAdapter"));
		MQe.alias("FastNetwork",        getArgument("FastNetwork",			"com.ibm.mqe.adapters.MQeTcpipHistoryAdapter"));
		MQe.alias("DefaultChannel",     getArgument("DefaultChannel",		"com.ibm.mqe.MQeChannel"));
		MQe.alias("AttributeKey_2", 	getArgument("AttributeKey_2",		"com.ibm.mqe.attributes.MQeSharedKey"));
		MQe.alias("AttributeKey_1", 	getArgument("AttributeKey_1",		"com.ibm.mqe.MQeKey"));
	} catch (Exception e) {
		_error("creating aliases ", e);
	}
}
/**
 * defQMRegistry - create the queue manager, ie create the
 * registry definitions. If there is an error, any newly
 * created definitions are deleted so that the registry is
 * left in its initial state.
 *
 * @param iniFields  A MQeFields object containing the configuration parameters.
 * @param queueDir      The directory for the queue store.
 *
 **/
private final boolean _createDefaultRegistry(MQeFields iniFields, String queueDir) {
	boolean success = false;
	// Configuration object
	MQeQueueManagerConfigure qMgrConfig = null;
	try {
		qMgrConfig = new MQeQueueManagerConfigure(iniFields, queueDir);
		//qMgrConfig.setChnlAttributeRuleName(MY_ATTRIBUTE_RULES);

		qMgrConfig.defineQueueManager();
		qMgrConfig.defineDefaultAdminQueue();
		qMgrConfig.defineDefaultAdminReplyQueue();
		qMgrConfig.defineDefaultSystemQueue();
		success = true;

	} catch (Exception e) {
		_error(" creating default registry", e);
	} finally {
		if (null != qMgrConfig) {
			qMgrConfig.close();
		}
	}
	return success;
}
/**
 * Create the mimimum ini fields neccessary for the creation of a queue manager
 *
 * Part of the programatic creation of a queue manager
 *
 * Specify queue manager name, and specify port. (if port is null, then a
 * listener is nt created).
 * the registry is created underneath the curren directory
 */
private final MQeFields _createIniFields(String qmName) {

	MQeFields iniFields = new MQeFields();
	try {
		// create registry section
		MQeFields section = _createRegistrySection();
		iniFields.putFields(MQeQueueManager.Registry, section);

		// create queue manager section
		section = new MQeFields();
		section.putAscii(MQeQueueManager.Name, qmName);
		iniFields.putFields(MQeQueueManager.QueueManager, section);

	} catch (Exception e) {
		_error("creating inifields ", e);
	}
	return iniFields;
}
/**
 * Create the queue manager
 *
 * Part of the programatic creation of a queue manager
 *
 * Specify queue manager name, and specify port. (if port is null, then a
 * listener is nt created).
 * the registry is created underneath the current directory
 *
 * @param iniFields  A MQeFields object containing the configuration parameters.
 */
private final boolean _createQueueManager(MQeFields iniFields) {
	boolean success = false;
	// create directory for queues store
	String queueDir = baseDirectory + "queues" + File.separator;
	boolean queueDirCreated = false;
	File dirFile = new File(queueDir);
	if (!dirFile.exists()) {
		_comment("Creating QueueManager queue directory");
		dirFile.mkdirs();
		queueDirCreated = true;
	}
	success = _createDefaultRegistry(iniFields, "MsgLog:" + queueDir);
	if (!success) {
		File dir = new File(baseDirectory);
		dir.delete();
	}
	return success;
}
/**
 * Create and return the registry section of the inifields
 */
protected MQeFields _createRegistrySection() {
	String key = getArgument(ARG_PIN);
	MQeFields section = new MQeFields();
	try {
		// create registry section
		if (null == key) {
			section.putAscii( com.ibm.mqe.registry.MQeRegistry.LocalRegType, "FileRegistry");
		} else {
			_comment("including pin ("+key+") in registry");
			section.putAscii( com.ibm.mqe.registry.MQeRegistry.LocalRegType, com.ibm.mqe.registry.MQeRegistry.PrivateRegistry);
			section.putAscii( MQeRegistry.PIN, key);
		}
		section.putAscii( com.ibm.mqe.registry.MQeRegistry.DirName, baseDirectory + "registry");
	} catch (Exception e) {
		_error("creating registry section of inifields ", e);
		section = null;
	}
	return section;
}
protected final void _error(String s) {
	System.out.println("Error: " + s);
}
protected final void _error(String s, Exception e) {
	System.out.println("Error: " + s + "{" + e.getMessage() + "}");
}
/**
 * Open the QueueManager section in the conguration data
 * Returns null if the open failed
 * @param sections  A MQeFields object containing the configuration parameters.
 */
private final MQeQueueManager _openQueueManager(MQeFields sections) {
	MQeQueueManager queueManager = new MQeQueueManager();
	MQeChannelManager channelManager = null;
	try {
		_comment("Opening QueueManager");
		if (!"NONE".equalsIgnoreCase(port)) {
			channelManager = new MQeChannelManager();
			channelManager.numberOfChannels(0);
			queueManager.setGlobalHashTable(channelManager.getGlobalHashtable());
		}
		queueManager.activate(sections);
	} catch (Exception e) {
		_comment("Could not open QueueManager: " + e.getMessage());
		queueManager = null;
	}
	if (null != queueManager && !"NONE".equalsIgnoreCase(port)) {
		try {
			channelListener = new MQeChannelListener("Network::" + port, "Network:", channelManager);
			channelListener.setTimer(0);
			_comment("started listener");
		} catch (Exception e) {
			_comment("Could not open listener: " + e.getMessage());

		}
	}
	return (queueManager);
}
protected final void _warning(String s) {
	System.out.println("Warning: " + s);
}
protected final void _warning(String s, Exception e) {
	System.out.println("Warning: " + s + "{" + e.getMessage() + "}");
}
/**
 * Adds a subscription to MQe Message Events
 *
 * This method registers an object as a listener to any MQeMessage events
 * generated by the queue specified in the queueName parameter. It is only
 * possible to add listeners to local queues.
 * Note: The listening object must implement the
 * MQeMessageListenerInterface. Events are processed by the event
 * handler methods specified in this interface.
 * A message filter consisting of message fields (for example MessageId or
 * Priority) may be specified so that the listening object only receives events
 * concerning messages that include the same fields as those specified. If no
 * fields are specified, events are triggered for all messages on the queue.
 *
 * published method
 *
 * @param listener (com.ibm.mqe.MQeMessageListener ) A reference to the subscribing object
 *
 * @param queueName A String containing the name of the queue from which the
 * listener wishes to receive events
 *
 * @param params (com.ibm.mqe.MQeField) object containing message fields. A
 * value of null means that the listener wishes to receive
 * events for all messages on the queue. Specifying an
 * MQeFields object containing message fields means that the
 * listener is only interested in events concerning messages
 * whose fields match those contained in the filter.
 *
 * @exception MQeException Except_QMgr_NotActive
 * @exception MQeException Except_QMgr_QdoesNotExist
 */
public void addMessageListener(MQeMessageListenerInterface objref, String queueName, MQeFields filter) throws Exception{
	myQM.addMessageListener(objref, queueName, filter);
}
/**
 * access to the standard API on the local QueueManager
 */
public final MQeEnumeration browseMessages(String queueManagerName, String queueName, MQeFields Filter, MQeAttribute attr, boolean justUID) throws Exception {
	return myQM.browseMessages(queueManagerName, queueName, Filter, attr, justUID);
}
/**
 * access to the standard API on the local QueueManager
 */
public final MQeEnumeration browseMessagesAndLock(String queueManagerName, String queueName, MQeFields Filter, MQeAttribute attr, long confirmid, boolean justUID) throws Exception {
	return myQM.browseMessagesAndLock(queueManagerName, queueName, Filter, attr, confirmid, justUID);
}
/**
 * access to the standard API on the local QueueManager
 */
public final void confirm(String queueManagerName, String queueName, long confirmID) throws Exception {
	//if (true) {
	//	myQM.confirm(queueManagerName, queueName, confirmID);
	//} else {
		System.out.println("********************** WARNING **********************");
		System.out.println("********************** WARNING **********************");
		System.out.println("** Confirm DISABLED for compatibility with old MQe **");
		System.out.println("********************** WARNING **********************");
		System.out.println("********************** WARNING **********************");
	//}
}
/**
 * access to the standard API on the local QueueManager
 */
public final void confirmGetMessage(String queueManagerName, String queueName, MQeFields filter) throws Exception {
	myQM.confirmGetMessage(queueManagerName, queueName, filter);
}
/**
 * access to the standard API on the local QueueManager
 */
public final void confirmPutMessage(String queueManagerName, String queueName, MQeFields filter) throws Exception {
	myQM.confirmPutMessage(queueManagerName, queueName, filter);
}
private final void delete(File f) throws java.io.IOException {
    //
    // If the file/directory does not exist, then nothing to do!
    //
    if (!f.exists())
        return;
    //
    // If the file is a directory, then recursively delete everything in it
    //
    if (f.isDirectory()) {
        String[] contents = f.list();
        for (int i = 0; i < contents.length; i++)
            delete(new File(f, contents[i]));
    }
    //
    // Now delete the file/directory itself.
    //
    //System.out.println("Deleting " + f);
    f.delete();
}
private final void deleteFile(String filename) {
    if (null != filename) {
        try {
            delete(new File(filename));
        } catch (IOException ioe) {
            System.err.println("Exception while deleting " + filename + " : " + ioe);
            ioe.printStackTrace(System.err);
        }
    }
}
/**
 * access to the standard API on the local QueueManager
 */
public final void deleteMessage(String queueManagerName, String queueName, MQeFields filter) throws Exception {
	myQM.deleteMessage(queueManagerName, queueName, filter);
}
public final String getArgument(String name) {
	return (String)arguments.get(name);
}
public final String getArgument(String name, String defaultValue) {
	String value = (String)arguments.get(name);
	if (null == value) {
		value = defaultValue;
	}
	return value;
}
protected String getBaseDirectory() {
	return baseDirectory;
}
/**
 * access to the standard API on the local QueueManager
 */
protected final MQeMsgObject getMessage(String queueManagerName, String queueName, MQeFields filter, MQeAttribute attr, long confirmID) throws Exception {
	return myQM.getMessage(queueManagerName, queueName, filter, attr, confirmID);
}
/**
 * access to the standard API on the local QueueManager
 */
protected final void putMessage(String queueManagerName, String queueName, MQeMsgObject msg, MQeAttribute attribute, long confirmID) throws Exception {
	myQM.putMessage(queueManagerName, queueName, msg, attribute, confirmID);
}
/**
 * removes a subscription to MQe Message Events
 *
 *
 * @exception MQeException Except_QMgr_NotActive
 * @exception MQeException Except_QMgr_QdoesNotExist
 *
 * access to the standard API on the local QueueManager
 */
public void removeMessageListener(MQeMessageListenerInterface objref, String queueName, MQeFields filter) throws Exception{
	myQM.removeMessageListener(objref, queueName, filter);
}
/**
 * channel timeout in milliseconds
 */
public final void setChannelTimeout(long newChnlTimeout)  {
	if (null != myQM) {
		myQM.setChannelTimeout(newChnlTimeout);
	}
}
public final boolean shutdown(Hashtable args) {
	try {
		_comment("Closing QM");
		if (null != myQM) {
			myQM.close();
		}
		if (null != channelListener) {
			channelListener.stop();
		}
		boolean clean = false;
		String str = (String) args.get(ARG_CLEANUP);
		if (null != str) {
			clean = str.toLowerCase().indexOf("true") > -1;
		}

		if (clean) {
			//System.out.println("Wait five seconds for things to go quiet before deleting directory");
			//Thread.sleep(5000);
			deleteFile(baseDirectory);
		}

	} catch (MQeException e) {
		if (e.code() != MQe.Except_QMgr_NotActive) {
			_comment("Shutting down. Exception thrown: ", e);
		}
	} catch (Exception e) {
		_comment("Shutting down. Exception thrown: ", e);
	}
	return true;
}
public final boolean shutdown(boolean removeDirectory) {
	try {
		_comment("Closing QM");
		if (null != myQM) {
			myQM.close();
		}
		if (null != channelListener) {
			channelListener.stop();
		}
		if (removeDirectory) {
			//System.out.println("Wait five seconds for things to go quiet before deleting directory");
			//Thread.sleep(5000);
			deleteFile(baseDirectory);
		}

	} catch (MQeException e) {
		if (e.code() != MQe.Except_QMgr_NotActive) {
			_comment("Shutting down. Exception thrown: ", e);
		}
	} catch (Exception e) {
		_comment("Shutting down. Exception thrown: ", e);
	}
	return true;
}
/**
 * .
 */
public boolean start(Hashtable args) {
	arguments = args;
	String str = getArgument(ARG_CLEANUP, "");
	boolean clean = str.toLowerCase().indexOf("true") > -1;
	
	queueManagerName = args.get(ARG_QUEUEMANAGER_NAME).toString();
	port = args.get(ARG_QUEUEMANAGER_PORT).toString();
		
	this.queueManagerName = queueManagerName;
	this.port = port;

	if (null == port) {
		port = "NONE";
	}

	_comment("QueueManager " + queueManagerName + " using port: " + port);

	baseDirectory = new File(".").getAbsolutePath() + File.separator + queueManagerName + File.separator;

	_createAliases();
	boolean success = false;
	if (clean) {
		deleteFile(baseDirectory);
		_comment("creating QueueManager"); 
		_createQueueManager(_createIniFields(queueManagerName));
	}


	myQM = _openQueueManager(_createIniFields(queueManagerName));
	
	if (null != myQM) {
		success = true;
		_comment("..Started queue manager: ");
	}
	return success;
}
/**
 * access to the standard API on the local QueueManager
 */
public final void triggerTransmission() throws Exception {
	myQM.triggerTransmission();
}
/**
 * access to the standard API on the local QueueManager
 */
public final void undo(String queueManagerName, String queueName, long confirmID) throws Exception {
	myQM.undo(queueManagerName, queueName, confirmID);
}
/**
 * access to the standard API on the local QueueManager
 */
public final void unlockMessage(String queueManagerName, String queueName, MQeFields filter) throws Exception {
	myQM.unlockMessage(queueManagerName, queueName, filter);
}
/**
   * Wait for message - waits for a mesaage to
   * arrive on the reply queue. (Uses MQe waitForMessage)
   * return message, or null if timed out
   */
public final MQeMsgObject waitForMessage(String queueManagerName, String queueName, 
											MQeFields match, MQeAttribute attribute, 
											long confirmID, int totalWaitTime) throws Exception {
	// try to get a message straight away
	MQeAdminMsg msg = (MQeAdminMsg)getMessage(queueManagerName, queueName, match, attribute, confirmID);
	while (null == msg && 0 < totalWaitTime) {
		try {
			Thread.sleep(1000);
			totalWaitTime -= 1000;
			msg = (MQeAdminMsg)getMessage(queueManagerName, queueName, match, attribute, confirmID);
		} catch (MQeException e) {
			if (e.code() != MQe.Except_Q_NoMatchingMsg) {
				// if the exception is 'no matching messzge then ignore it.  This
				// will result in a null return value.
				// Rethrow all other exceptions
				throw e;
			}
		} catch (InterruptedException e) {
		}
	}
	return msg;
}
}
