package glyph;

import java.awt.*;
/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

/**
 * This class represents a line between two glyphs.
 * Creation date: (2/7/02 8:27:25 AM)
 * @author: 
 */
class LineGlyph extends Object {
	private static final int ARROWSIZE = 7;
	private static final double ARROWFLARE = 0.6;
	private static final int DIAMONDSIZE = 8;
	private static final double DIAMONDFLARE = 0.6;
		
	private AreaGlyph srcGlyph;
	private AreaGlyph dstGlyph;
	
	private View view = null;
	private boolean changed = true;

	private int labelRectHeight;
	private int labelRectWidth;
	private int textX;
	private int textY;
	private int textHeight;
	private int labelRectY;
	private int labelRectX;

	private int[][] srcSymbolCoords;
	private int[][] dstSymbolCoords;
	Point start;
	Point end;
	Line line;
	boolean masked = false;
/**
 * package private constructor
 */
LineGlyph(Line line, AreaGlyph source, AreaGlyph dest) {
	super();
	this.line = line;
	srcGlyph = source;
	
	dstGlyph = dest;
	srcGlyph._addLine(this);
	dstGlyph._addLine(this);
}
private final int[][] _arrowHeadVertices(Point src, Point dst) {
	// p1 is target point, p2 is 'tail'
	int p1x = dst.x;
	int p1y = dst.y;
	int p2x = src.x;
	int p2y = src.y;
	int x[] = new int[4];
	int y[] = new int[4];
	
	for (int i = 0; i < 4; i++) {
		x[i] = p1x; y[i] = p1y;
	}

	int deltaX = p2x - p1x;
	int deltaY = p2y - p1y;
	// calculate length of line between points
	double lineLength = java.lang.Math.sqrt((deltaX * deltaX) + (deltaY * deltaY));
	// what proportion of this do we want?
	double ratio = (double) ARROWSIZE / lineLength; // small arrow length is 20?
	x[2] += (int)(ratio * deltaX);
	y[2] += (int)(ratio * deltaY);

	/* this is the 'back point' of arrow on line. We wish to 
	 * construct an imaginary line through this orthogonal to
	 * the original line and offset along it in both directions
	 * to determine the outer points of the arrow
	 */
	x[1] = x[3] = p1x + (int)(ratio * deltaX * 2);
	y[1] = y[3] = p1y + (int)(ratio * deltaY * 2);

	/* with an orthogonal line the x and y deltas are exchanged
	 * but the length will remain the same, so the ratios will 
	 * be the same but we wish to reduce arrow angle
	 */
	ratio *= ARROWFLARE;
	int temp = deltaX;
	deltaX = deltaY;
	deltaY = temp;
	x[1] += (int)(ratio * deltaX);
	y[1] -= (int)(ratio * deltaY);
	x[3] -= (int)(ratio * deltaX);
	y[3] += (int)(ratio * deltaY);
	return new int[][]{x,y};
}
private final int[][] _bluntArrowHeadVertices(Point src, Point dst) {
	// p1 is target point, p2 is 'tail'
	int p1x = dst.x;
	int p1y = dst.y;
	int p2x = src.x;
	int p2y = src.y;
	int x[] = new int[4];
	int y[] = new int[4];
	
	for (int i = 0; i < 4; i++) {
		x[i] = p1x; y[i] = p1y;
	}

	int deltaX = p2x - p1x;
	int deltaY = p2y - p1y;
	// calculate length of line between points
	double lineLength = java.lang.Math.sqrt((deltaX * deltaX) + (deltaY * deltaY));
	// what proportion of this do we want?
	double ratio = (double) ARROWSIZE / lineLength; // small arrow length is 20?

	
	/* this is the 'back point' of arrow on line. We wish to 
	 * construct an imaginary line through this orthogonal to
	 * the original line and offset along it in both directions
	 * to determine the outer points of the arrow
	 */
	x[1] = x[2] = x[3] = p1x + (int)(ratio * deltaX * 2);
	y[1] = y[2] = y[3] = p1y + (int)(ratio * deltaY * 2);

	/* with an orthogonal line the x and y deltas are exchanged
	 * but the length will remain the same, so the ratios will 
	 * be the same but we wish to reduce arrow angle
	 */
	ratio *= ARROWFLARE;
	int temp = deltaX;
	deltaX = deltaY;
	deltaY = temp;
	x[1] += (int)(ratio * deltaX);
	y[1] -= (int)(ratio * deltaY);
	x[3] -= (int)(ratio * deltaX);
	y[3] += (int)(ratio * deltaY);
	return new int[][]{x,y};
}
/**
 * precalculate the size and layout
 */
void _calculate(Graphics graphics, Rectangle bounds) {
	if (null == srcGlyph || null == dstGlyph) {
		detach();
		return;
	}
	if (srcGlyph.isAttachedTo(dstGlyph)) {
		// remove no-op conditions
		return;
	}
	if (!isVisible()) {
		return;
	}
	
	
	// calculate
	start = srcGlyph.getIntersectPointFrom(dstGlyph.getCentre());
	end = dstGlyph.getIntersectPointFrom(srcGlyph.getCentre());

	
	String[] body = line.getBody(); 
	if (null != body) {
		// calculate
		int labelCentreX = (start.x + start.x + end.x) / 3;
		int labelCentreY = (start.y + start.y + end.y) / 3;
		FontMetrics fm = graphics.getFontMetrics();
		labelRectHeight = fm.getHeight() * body.length;
		labelRectWidth = 0;
		for (int i = 0; i < body.length; i++){
			int w = fm.stringWidth(body[i]);
			if (w > labelRectWidth) {
				labelRectWidth = w;
			}
		}
		textX = labelCentreX - (labelRectWidth/2);
		labelRectY = labelCentreY - (labelRectHeight /2);
		labelRectX = textX-(labelRectHeight/2);
		textY = labelRectY+fm.getAscent()+fm.getLeading();
		
	}
	
	// calculate
	switch (line.getSourceSymbol()) {
		case Line.SYMBOL_HOLLOW_ARROW:
		case Line.SYMBOL_ARROW:
			srcSymbolCoords = _arrowHeadVertices(end,start);
			break;
		case Line.SYMBOL_HOLLOW_BLUNT_ARROW:
		case Line.SYMBOL_BLUNT_ARROW:
			srcSymbolCoords = _bluntArrowHeadVertices(end,start);
			break;
		case Line.SYMBOL_DIAMOND:
		case Line.SYMBOL_HOLLOW_DIAMOND:
			srcSymbolCoords = _diamondVertices(end,start);
			break;
	}
	switch (line.getDestinationSymbol()) {
		case Line.SYMBOL_HOLLOW_ARROW:
		case Line.SYMBOL_ARROW:
			dstSymbolCoords = _arrowHeadVertices(start, end);
			break;
		case Line.SYMBOL_HOLLOW_BLUNT_ARROW:
		case Line.SYMBOL_BLUNT_ARROW:
			dstSymbolCoords = _bluntArrowHeadVertices(start, end);
			break;
		case Line.SYMBOL_DIAMOND:
		case Line.SYMBOL_HOLLOW_DIAMOND:
			dstSymbolCoords = _diamondVertices(start, end);
			break;
	}
	changed = false;
}
/**
 * the receiver has changed in a way that requires recalculation
 * before the next repaint
 */	
final void _changed() {
	changed = true;
}
private final int[][] _diamondVertices(Point src, Point dst) {
	// p1 is target point, p2 is 'tail'
	int p1x = dst.x;
	int p1y = dst.y;
	int p2x = src.x;
	int p2y = src.y;
	int x[] = new int[4];
	int y[] = new int[4];
	
	for (int i = 0; i < 4; i++) {
		x[i] = p1x; y[i] = p1y;
	}

	int deltaX = p2x - p1x;
	int deltaY = p2y - p1y;
	// calculate length of line between points
	double lineLength = java.lang.Math.sqrt((deltaX * deltaX) + (deltaY * deltaY));
	// what proportion of this do we want?
	double ratio = (double) DIAMONDSIZE / lineLength; // small arrow length is 20?
	x[2] += (int)(ratio * deltaX * 2);
	y[2] += (int)(ratio * deltaY * 2);

	/* this is the 'mid point' of diamond on line. We wish to 
	 * construct an imaginary line through this orthogonal to
	 * the original line and offset along it in both directions
	 * to determine the outer points of the diamond
	 */
	x[1] = x[3] = p1x + (int)(ratio * deltaX);
	y[1] = y[3] = p1y + (int)(ratio * deltaY);

	/* with an orthogonal line the x and y deltas are exchanged
	 * but the length will remain the same, so the ratios will 
	 * be the same but we wish to reduce arrow angle
	 */
	ratio *= DIAMONDFLARE;
	int temp = deltaX;
	deltaX = deltaY;
	deltaY = temp;
	x[1] += (int)(ratio * deltaX);
	y[1] -= (int)(ratio * deltaY);
	x[3] -= (int)(ratio * deltaX);
	y[3] += (int)(ratio * deltaY);

	return new int[][]{x,y};
}
/**
 * Draw a line from the receiver to the specified glyph, decorating
 * the ends of the line with the specified symbol, and labelling the
 * centre of the line with the specified text (null = no lable).
 *
 * If the glyph at either end of the line is still attached to its
 * parent glyph, then the line is drawn to the parent glyph.  This 
 * applies even if an arbitrary number of parent levels is involved.
 *
 * The exception to this rule is if the two glyphs are attached to
 * each other, either directly, or via an arbitrary number of
 * parents to a common super-parent.
 */
final void _draw(DrawContext graphics) {
	
	if (!isVisible()
		|| null == start
		|| null == srcGlyph 
		|| null == dstGlyph 
		|| srcGlyph.isAttachedTo(dstGlyph)
		|| !srcGlyph.isSetVisible() 
		|| !dstGlyph.isSetVisible()
		|| _isMasked()) {
		// remove no-op conditions
			return;
	}
	int x1 = start.x;
	int y1 = start.y;
	int x2 = end.x;
	int y2 = end.y;
	
	//Color oldColor = graphics.getColor();



	switch (line.getSourceSymbol()) {
		case Line.SYMBOL_ARROW:
		case Line.SYMBOL_BLUNT_ARROW:
		case Line.SYMBOL_DIAMOND:
			graphics.drawPolygon(
				srcSymbolCoords[0], 
				srcSymbolCoords[1], 
				srcSymbolCoords[0].length,
				Color.black, Color.black);
			x1=srcSymbolCoords[0][2];
			y1=srcSymbolCoords[1][2];
			break;
		case Line.SYMBOL_HOLLOW_ARROW:
		case Line.SYMBOL_HOLLOW_BLUNT_ARROW:
		case Line.SYMBOL_HOLLOW_DIAMOND:
			x1=srcSymbolCoords[0][2];
			y1=srcSymbolCoords[1][2];
			graphics.drawPolygon(
				srcSymbolCoords[0], 
				srcSymbolCoords[1], 
				srcSymbolCoords[0].length,
				Color.black, Color.white);
			break;
	}
	
	
	switch (line.getDestinationSymbol()) {
		case Line.SYMBOL_ARROW:
		case Line.SYMBOL_BLUNT_ARROW:
		case Line.SYMBOL_DIAMOND:
			x2=dstSymbolCoords[0][2];
			y2=dstSymbolCoords[1][2];
			graphics.drawPolygon(
				dstSymbolCoords[0], 
				dstSymbolCoords[1], 
				dstSymbolCoords[0].length,
				Color.black, Color.black);
			break;
		case Line.SYMBOL_HOLLOW_ARROW:
		case Line.SYMBOL_HOLLOW_BLUNT_ARROW:
		case Line.SYMBOL_HOLLOW_DIAMOND:
			graphics.drawPolygon(
				dstSymbolCoords[0], 
				dstSymbolCoords[1], 
				dstSymbolCoords[0].length,
				Color.black, Color.white);
			x2=dstSymbolCoords[0][2];
			y2=dstSymbolCoords[1][2];
			break;
	}
	
	graphics.drawLine(x1,y1,x2,y2);

	String[] body = line.getBody(); 
	if (null != body) {
		//graphics.setColor(Color.white);
		//graphics.fillRect(labelRectX,labelRectY, labelRectWidth+labelRectHeight, labelRectHeight);
		graphics.drawRect(
			labelRectX,labelRectY, 
			labelRectWidth+labelRectHeight, labelRectHeight,
			Color.white, Color.white);
		int y = textY;
		for (int i = 0; i < body.length; i++){
			graphics.drawString(body[i], textX, y);
			y += textHeight;
		}
	}
}
private final boolean _isMasked() {
	boolean masked = false;
	/*
	java.util.Vector siblingLines = srcGlyph.attachedLines();
	for (int i = 0; i < siblingLines.size(); i++){
		LineGlyph sibling = (LineGlyph)siblingLines.elementAt(i);
		if (sibling.getImportance() > getImportance()) {
			// only bother checking if sibling is more important than me
			// we need to compare current logical destinatiosn - ie the glyphs between
			AreaGlyph myLogicalDst
			
			if (sibling.getDestinationGlyph() == getDestinationGlyph()
				&& sibling.getSourceGlyph() == getSourceGlyph()) {
					masked = true;
			} else if (sibling.getDestinationGlyph() == getSourceGlyph()
				&& sibling.getSourceGlyph() == getDestinationGlyph()) {
					masked = true;
			}
		}
	}
	*/
	return masked;
}
/**
 * detach the line and render it useless
 */
public final void detach() {
	getGlyphView()._removeLine(this);
	if (null != srcGlyph) {
		srcGlyph._removeLine(this);
	}
	if (null != dstGlyph) {
		dstGlyph._removeLine(this);
	}
	srcGlyph = null;
	dstGlyph = null;
}
final AreaGlyph getDestinationGlyph() {
	return dstGlyph;
}
public View getGlyphView() {
	if (null == view && null != srcGlyph) {
		view = srcGlyph.getGlyphView();
	}
	if (null == view && null != dstGlyph) {
		view = dstGlyph.getGlyphView();
	}
	if (null == view) {
		System.out.println("Null view!");
		new Exception().printStackTrace();
	}

	return view;
}
	/**
	 * reply importance of line.  Higher importance lines
	 * mask lower importance lines.  Default is 10.
	 */
	public final int getImportance() {
		return line.getImportance();
	}
final AreaGlyph getSourceGlyph() {
	return srcGlyph;
}
public View getView() {
	return view;
}
/**
 * reply true if the receiver is still attached
 */
final boolean isAttached() {
	return null != srcGlyph;
}
/**
 * reply true if the receiver is still attached
 */
final boolean isVisible() {
	return line.isVisible();
}
public String toString() {
	return "LineGlyph: " 
			//+ line.getBody() 
			+ "("
			+ srcGlyph
			+ " --> "
			+ dstGlyph
			+ ")";
}
}
